﻿// -----------------------------------------------------------------------
//  <copyright file="MakoDll.cs" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------


using System;
using System.Runtime.InteropServices;

namespace Mako.Wrapper
{
    public enum eFileFormat
    {
        eFFPDF,
        eFFXPS,
        eFFPS,
        eFFSVG,
        eFFPCL5,
        eFFPCLXL,
        eFFUnknown
    }

    public static class MakoDll
    {
        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool fnInitialize([MarshalAs(UnmanagedType.LPStr)] string libraryLocation);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnShutdown();

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnOpenAssembly([MarshalAs(UnmanagedType.LPStr)] string assemblyFile, 
            eFileFormat format, ref IntPtr assemblyHandle);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnCloseAssembly(IntPtr assemblyHandle);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern void fnSaveAssembly(IntPtr assemblyHandle, 
            [MarshalAs(UnmanagedType.LPStr)] string filePath,
            [MarshalAs(UnmanagedType.LPArray, ArraySubType = UnmanagedType.LPStr, SizeParamIndex = 1)] string[] outputParams, int parameterCount);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern void fnSaveAssemblyAsPdf(IntPtr assemblyHandle,
            [MarshalAs(UnmanagedType.LPStr)] string filePath, bool incremental);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr fnOpenDocument(IntPtr assemblyHandle, int documentIndex);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern void fnCloseDocument(IntPtr documentHandle);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern int fnGetPageCount(IntPtr documentHandle);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern double fnGetPageWidth(IntPtr documentHandle, int pageIndex);

        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl)]
        public static extern double fnGetPageHeight(IntPtr documentHandle, int pageIndex);
        
        [DllImport("libMakoWrapperDll.so", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern void fnSaveThumbnail
        (
            IntPtr documentHandle,
            int pageIndex, 
            [MarshalAs(UnmanagedType.LPStr)] string filePath
        );
    }

    public class MakoDllException : Exception
    {
        public MakoDllException(string message, int errorCode)
            : base($"An unexpected error occurred in Mako: {message}, error: {errorCode}")
        {
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MakoSize
    {
        public double width;
        public double height;
    }
}
