// -----------------------------------------------------------------------
//  <copyright file="MakoWrapperDll.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

//#include "stdafx.h"
//#include <Windows.h>
//#include <direct.h>
//#include <objbase.h>
#include <iostream>

#include <jawsmako/jawsmako.h>
#include <jawsmako/pdfinput.h>
#include <jawsmako/pdfoutput.h>
#include <jawsmako/transforms.h>
#include <WrapperLib/Types.h>
#include <WrapperLib/MakoAssembly.h>
#include <WrapperLib/PageRenderer.h>

//#include "GLock.h"
#include "MakoWrapperDll.h"

using namespace EDL;
using namespace JawsMako;
using std::string;

IJawsMakoPtr gMako;

//static GCRITICAL_SECTION gCriticalSection;

MAKOWRAPPER_API bool fnTestMarshalling(uint32 testInt, char* message )
{
    std::cerr << "The number you thought of was " << testInt << " " << message;

    return true;
}

MAKOWRAPPER_API bool fnInitialize(char* libraryLocation)
{
    try
    {
        const U8String location(libraryLocation);
        std::cerr << libraryLocation;

        // Create the Mako instance
        gMako = IJawsMako::create(location);

        IJawsMako::enableAllFeatures(gMako);
    }
    catch (IEDLError &)
    {
        return false;
    }

    return true;
}

MAKOWRAPPER_API void fnShutdown()
{
    gMako = nullptr;
}

MAKOWRAPPER_API MakoResult fnOpenAssembly(char* filePath, eFileFormat format, MakoHandle* assemblyHandle)
{
    U8String documentFile(filePath);
    std::cerr << "assembly file path(s):" << documentFile;
    IDocumentAssemblyPtr assembly;

    *assemblyHandle = nullptr;

    try
    {
        IInputPtr fileInput = IInput::create(gMako, format);
        assembly = fileInput->open(documentFile);
    }
    catch (IEDLError& err)
    {
        return err.getErrorCode();
    }

    // Lock the following section for thread safe
    //GLock lock(&gCriticalSection);

    MakoAssembly *makoAssembly = new MakoAssembly(gMako, assembly, documentFile);
    *assemblyHandle = reinterpret_cast<MakoHandle>(makoAssembly);

    return 0;
}

MAKOWRAPPER_API MakoResult fnOpenAssemblyAsPdf(char* filePath, MakoHandle* assemblyHandle)
{
    U8String documentFile(filePath);
    IDocumentAssemblyPtr assembly;

    *assemblyHandle = nullptr;

    try
    {
        IPDFInputPtr pdfInput = IPDFInput::create(gMako);
        assembly = pdfInput->open(documentFile);
    }
    catch (IEDLError& err)
    {
        return err.getErrorCode();
    }

    // Lock the following section for thread safe
    //GLock lock(&gCriticalSection);

    MakoAssembly *makoAssembly = new MakoAssembly(gMako, assembly, documentFile);
    *assemblyHandle = reinterpret_cast<MakoHandle>(makoAssembly);

    return 0;
}

MAKOWRAPPER_API void fnCloseAssembly(MakoHandle assemblyHandle)
{
    MakoAssembly* assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    delete assembly;
}

MAKOWRAPPER_API void fnSaveAssembly(MakoHandle assemblyHandle, char* filePath, char** outputParams, int parameterCount)
{
    MakoAssembly *assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    CEDLSysStringVect parameters;
    for (auto i = 0; i < parameterCount; i++)
    {
        parameters.append(U8String(outputParams[i]));
    }
    assembly->save(U8String(filePath), parameters);
}

MAKOWRAPPER_API void fnSaveAssemblyAsPdf(MakoHandle assemblyHandle, char* filePath, bool incremental)
{
    MakoAssembly *assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    assembly->saveAsPdf(U8String(filePath), incremental);
}

MAKOWRAPPER_API MakoHandle fnOpenDocument(MakoHandle assemblyHandle, int documentIndex)
{
    MakoAssembly *assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    MakoDocument* document = assembly->openDocument(documentIndex);

    return reinterpret_cast<MakoHandle>(document);
}

MAKOWRAPPER_API void fnCloseDocument(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    delete document;
}

MAKOWRAPPER_API int fnGetPageCount(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getPageCount();
}

MAKOWRAPPER_API double fnGetPageWidth(MakoHandle documentHandle, int pageIndex)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getPageWidth(pageIndex);
}

MAKOWRAPPER_API double fnGetPageHeight(MakoHandle documentHandle, int pageIndex)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getPageHeight(pageIndex);
}

MAKOWRAPPER_API void fnGetThumbnail(MakoHandle documentHandle, int pageIndex, char* filePath)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    IPagePtr page = document->getPage(pageIndex);
    PageRenderer Renderer = PageRenderer(gMako, page);
    const IDOMImagePtr renderedImage = Renderer.Render();
    IDOMPNGImage::encode(gMako, renderedImage, IOutputStream::createToFile(gMako, filePath));
}