/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/*
*  edlallocator.h
*  
*  A memory allocator, similar to the STL Allocator.
*
*  Useful for defining who has ownership of memory allocation and deallocation.
*  This is primarily for the windows platform, to handle conflicts between execution and DLL memory spaces.
*
*/

#ifndef EDLALLOCATOR_H
#define EDLALLOCATOR_H

#include <stdlib.h>
#include <edl/edltypes.h>
#include <edl/edlnamespaces.h>

_BEGIN_EDL_NAMESPACE

class EDL_API EDLAllocator
{
public:
    
    typedef void *(*AllocatorFn)(size_t n);
    typedef void (*DeallocatorFn)(void *p);
    typedef void *(*ReallocateFn)(void *p, size_t newSize);
    
    EDLAllocator();
    
    virtual ~EDLAllocator()
    {}
    
    EDLAllocator(AllocatorFn allocator, DeallocatorFn deallocator, ReallocateFn reallocator)
    {
        m_allocator = allocator;
        m_deallocator = deallocator;
        m_reallocator = reallocator;
    }
    
    void *allocate(size_t size)                     { return m_allocator(size); }
    void deallocate(void *p)                        { m_deallocator(p); }
    void *reallocate(void *p, size_t newSize)       { return m_reallocator(p, newSize); }
    
private:
    
    AllocatorFn     m_allocator;
    DeallocatorFn   m_deallocator;
    ReallocateFn    m_reallocator;
};


_END_EDL_NAMESPACE

#endif /* EDLALLOCATOR */

