/*
* Copyright (C) 2012 Global Graphics Software Ltd. All rights reserved.
*
*/

/**
* @file edlquartz.h
* @ingroup IFactory
* @brief A renderer that allows painting of EDL DOM into a Quartz2D Core Graphics
*        context. Currently only XPS-compatible DOM is supported.
*        The renderer is reentrant and can be used on multiple threads, providing
*        that the destination context supports this. The renderer also caches
*        certain objects to improve performance of repeat renders.
*/

#include "TargetConditionals.h"

#if TARGET_OS_IPHONE || TARGET_IPHONE_SIMULATOR
#import <UIKit/UIKit.h>
#else
#import <Cocoa/Cocoa.h>
#endif
#include <edl/isession.h>
#include <edl/edltypes.h>
#include <edl/idomnode.h>
#include <edl/edlgeom.h>

_USING_EDL_NAMESPACE

@interface EDLQuartzRenderer : NSObject
{
@private
    void *renderPriv;
}

/// Initialize the renderer
- (id) initWithSession:(ISessionPtr) session;

/// Draw a node to the context.
- (BOOL) drawNode:(IDOMNodePtr &) node toContext:(CGContextRef)context;

/// Release memory in caches. Useful when the system is running low on memory.
- (void) releaseCaches;

@end

