/*
* Copyright (C) 2012-2019 Global Graphics Software Ltd. All rights reserved.
*/

#ifndef __EDLIDOMHASHABLE_H_DECLS__
#define __EDLIDOMHASHABLE_H_DECLS__

/**
* @file     idomhashable.h
* @ingroup  IDom
* @brief    Abstract interface for objects that can be hashed.
*/

#include <edl/edlnamespaces.h>
#include <edl/iedlobject.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class   IDOMHashable
* @ingroup IDom
* @brief   Abstract interface for EDL objects that may be hashed.
* @details Useful for determining a fast, 64 bit hash of objects that support this
*          interface. Not guaranteed to be completely unique, but collisions are
*          unlikely.
*/
class IDOMHashable
{
public:
    /**
    * @brief    Retrieve a hash for this object.
    * @param    hash A reference to receive the hash.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool hash(uint64 &hash) = 0;
};
#define edlobj2IDOMHashable(src) dynamic_cast<IDOMHashable *>((IEDLObject *) src)

_END_EDL_DOM_NAMESPACE

#endif

