/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file iedlcollection.h
 * @ingroup IController
 * @brief EDL "collection" interface
 */

#ifndef __IEDLCOLLECTION_H__
#define __IEDLCOLLECTION_H__

#include <edl/ircobject.h>
#include <edl/iedlenum.h>
#include <edl/idomid.h>
#include <edl/smartptr.h>

_BEGIN_EDL_NAMESPACE

template <typename T>
class IEDLCollection : public IRCObject
{
public:
        virtual CSmartPtr<IInputEnumRC <T> > getEnumerator() = 0;
        virtual uint32 count() const = 0;
        virtual void clear() = 0;
        virtual bool insertBefore(const CSmartPtr<IInputEnumRC <T> > &enumerator, const T &elm) = 0;
        virtual bool add(const T &elm) = 0;
        virtual bool erasePrevious(const CSmartPtr<IInputEnumRC <T> > &enumerator) = 0;
};

typedef IEDLCollection<EDLString>           IEDLStringCollection;
DECL_SMART_PTR (IEDLStringCollection);

typedef CSmartPtr<IEDLCollection<double> >  CDoubleCollectionPtr;
typedef CSmartPtr<IEDLCollection<FPoint> >  CFPointCollectionPtr;
typedef CSmartPtr<IEDLCollection<FRect> >   CFRectCollectionPtr;
typedef CSmartPtr<IEDLCollection<DOMid> >   CDOMidCollectionPtr;

_END_EDL_NAMESPACE

#endif /* __IEDLCOLLECTION_H__ */
