/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file iedlenum.h
 * @ingroup IController
 * @brief EDL iterator template classes designed to allow
 * iteration over the contents of a collection.
 * Then a set of typedefs that provide collections of
 * doubles, FPoints, FRects, EDLStrings and EDLSysStrings
 * that can then be iterated over.
 */

#ifndef __IEDLENUM_H__
#define __IEDLENUM_H__

#include <edl/ircobject.h>
#include <edl/idomid.h>
#include <edl/edlstring.h>
#include <edl/edlgeom.h>
#include <edl/edltypes.h>

_BEGIN_EDL_NAMESPACE

/**
 * @class IInputEnum<typename T>
 * @brief Iterator template class to allow iteration
 * over a collection of instances of type \<T\>
 */

template <typename T>
class IInputEnum
{
public:
    virtual uint32 count() const = 0;
    virtual void reset() = 0;
    virtual bool getNext(T* next ) = 0;
};

/**
 * @class IInputEnumRC<typename T>
 * @brief Reference-counted iterator template class to allow iteration
 * over a collection of instances of type \<T\>
 */

template <typename T>
class IInputEnumRC : public IRCObject
{
public:
    virtual uint32 count() const = 0;
    virtual void reset() = 0;
    virtual bool skip(uint32 nSkip) = 0;
    virtual bool getNext(T* next ) = 0;
};

typedef IInputEnumRC<double> IDoubleCollectionEnum;
DECL_SMART_PTR(IDoubleCollectionEnum);

typedef IInputEnumRC<FPoint> IFPointCollectionEnum;
DECL_SMART_PTR(IFPointCollectionEnum);

typedef IInputEnumRC<FRect> IFRectCollectionEnum;
DECL_SMART_PTR(IFRectCollectionEnum);

typedef IInputEnumRC<EDLString> IEDLStringCollectionEnum;
DECL_SMART_PTR(IEDLStringCollectionEnum);

typedef IInputEnumRC<EDLSysString> IEDLSysStringCollectionEnum;
DECL_SMART_PTR(IEDLSysStringCollectionEnum);

typedef IInputEnumRC<DOMid> IDOMidCollectionEnum;
DECL_SMART_PTR(IDOMidCollectionEnum);

_END_EDL_NAMESPACE

#endif /* __IEDLENUM_H__ */
