/*
 * Copyright (C) 2008 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file iedltree.h
 * @ingroup IController
 * @brief EDL "tree" interface
 */

#ifndef __IEDLTREE_H__
#define __IEDLTREE_H__

#include <edl/smartptr.h>
#include <edl/iedlobject.h>
#include <edl/edltypes.h>

_BEGIN_EDL_NAMESPACE

template <typename T>
class IEDLTreeNode : public IEDLObject
{
public:
    virtual bool getData(T &data) const = 0;
    virtual void setData(const T &data) = 0;

    virtual uint32 getChildrenCount() const = 0;
    virtual CSmartPtr<IEDLTreeNode<T> > getChild(uint32 index) = 0;
    virtual IEDLTreeNode<T> * getParent() = 0;

    virtual bool insertChildAt(uint32 index, const CSmartPtr<IEDLTreeNode<T> > &child) = 0;
    virtual bool appendChild(const CSmartPtr<IEDLTreeNode<T> > &child) = 0;
    virtual CSmartPtr<IEDLTreeNode<T> > detachChild(uint32 index) = 0;

    virtual bool deleteChild(uint32 index) = 0;
    virtual void remove() = 0;

};

template <typename T>
class IEDLTree : public IEDLObject
{
public:
    virtual CSmartPtr<IEDLTreeNode<T> > getRoot() = 0;
};

_END_EDL_NAMESPACE

#endif /* __IEDLTREE_H__ */
