/*
 * Copyright (C) 2014-2019 Global Graphics Software Ltd. All rights reserved.
 */

#ifndef JAWSMAKO_PCLXLOUTPUT_H
#define JAWSMAKO_PCLXLOUTPUT_H

/**
 * @file     pclxloutput.h
 * @ingroup  JawsMako
 * @brief    JawsMako PCLXL Output.
 */

#include <jawsmako/jawsmako.h>

namespace JawsMako
{
    using namespace EDL;
    
    class IPCLXLOutput;
    DECL_SMART_PTR(IPCLXLOutput);
    
    /**
     * @ingroup  JawsMako
     * @class    IPCLXLOutput
     * @brief    Interface for the PCLXL IOutput class
     */
    class IPCLXLOutput : public IOutput
    {
    public:
        virtual ~IPCLXLOutput() {}
        
        /**
         * @brief   Create a PCL5 Output instance.
         */
        static IPCLXLOutputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
         * PCLXL Output-specific configuration
         */
        
        /**
         * @brief Set the PCLXL resolution
         *        The default is 600.
         *        Equivalent to calling setParameter() with the parameter name
         *        "Resolution".
         */
        virtual void setResolution(float resolution) = 0;
        
        /**
         * @brief Set whether the output stream should be opened or not.
         *
         *        If true, the output stream will be opened and closed by the
         *        PCLXL output. If false, the stream will assumed to be opened
         *        and will not be closed.
         *
         *        Setting this to false allows the PCLXL stream to be written
         *        to an existing stream or channel.
         *
         *        The default is true.
         *
         *        Equivalent to calling setParameter() with the parameter name
         *        "OpenStream", the value "true" or "false".
         */
        virtual void setOpenStream(bool open) = 0;
        
        /**
         * @brief Set whether or not a PJL header or end of job should be emitted.
         *
         *        The default is true.
         *
         *        Equivalent to calling setParameter() with the parameter name
         *        "EmitPJL", the value "true" or "false".
         */
        virtual void setEmitPjl(bool emitPjl) = 0;

    };
#define obj2IPCLXLOutput(obj) IPCLXLOutputPtr(dynamic_cast<IPCLXLOutput *>((IRCObject *) obj), true)
}

#endif

