/*
* Copyright (C) 2014-2016 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef _JAWSMAKO_SKIARENDERER_H_
#define _JAWSMAKO_SKIARENDERER_H_

/**
* @file     skiarenderer.h
* @ingroup  JawsMako
* @brief    A Renderer for XPS compatible DOM using Skia.
*
* This header describes the ISkiaRenderer interface.
*/

#include <jawsmako/jawsmako.h>

#include "SkCanvas.h"
#include "SkBitmap.h"

namespace JawsMako
{
    using namespace EDL;

    class ISkiaRenderer;
    DECL_SMART_PTR(ISkiaRenderer);

    /**
    * @ingroup  JawsMako
    * @class    ISkiaRenderer
    * @brief    A renderer that can paint XPS compatible DOM into a Skia
    *           canvas using the Skia API.
    */
    class ISkiaRenderer : public IRCObject
    {
      public:
        virtual ~ISkiaRenderer() {}

        /**
        * @brief   Create a Skia Renderer Instance.
        *
        *          As some data will be cached, it is best to reuse a single
        *          instance when repeatedly rendering the same content or
        *          content from the same document.
        */
        static JAWSMAKO_API ISkiaRendererPtr create(const IJawsMakoPtr &jawsMako);

        /**
        * @brief   Render the given node into the Skia canvas.
        *
        *          It is safe for multiple threads to call this member at the
        *          same time, but care should be taken to ensure that the
        *          target canvas environment supports this.
        */
        virtual void drawNode(const IDOMNodePtr &node, SkCanvas *canvas) = 0;
        
        /**
        * @brief   Flush all caches used by the renderer.
        *
        *          This is required if the renderer has previously been used to
        *          paint content from a file that no longer exists as some of the
        *          cached results may point to content within such files. It is
        *          advisable to invoke this function if such a file is deleted or
        *          moved.
        *
        *          Secondarily, this may provide temporary memory relief as
        *          cached objects are released.
        */
        virtual void flushCaches() = 0;
     };
}

#endif
